### 实验十五 OneNet物联网连接

#### OneNet云平台创捷产品

1. 进入旧版控制台

   ![image-20221206143517043](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十五 OneNet物联网连接\添加)

   

2.创建产品

进入MQTT物联网平台

![image-20221206143532087](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十五 OneNet物联网连接\物联网.png)

点击右上角添加产品

![image-20221206143639866](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十五 OneNet物联网连接\物联网1.png)

设置信息后确定，然后立即添加设备，编辑一个名称即可

创建完成后点击详情可以看到相关信息

![image-20221206143818294](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十五 OneNet物联网连接\设备1.png)

在设备主页可以看到产品的ID和用户ID，还有设备的相关信息，记录这些

然后根据这个在线网址生成加密token： [https://open.iot.10086.cn/doc/mqtt/book/manual/auth/tool.html](https://links.jianshu.com/go?to=https%3A%2F%2Fopen.iot.10086.cn%2Fdoc%2Fmqtt%2Fbook%2Fmanual%2Fauth%2Ftool.html)

然后我们还可以根据需要订阅相关的主题：[概述_开发者文档_OneNET (10086.cn)](https://open.iot.10086.cn/doc/mqtt/book/device-develop/topics/introduce.html)

#### 代码讲解

全部准备完毕后烧录以下程序即可连接OneNet云平台并且向里面发送温湿度数据

![image-20221206144043705](E:\派哟项目\Hexbot项目\HEXBot_使用手册\ESP32使用手册\实验十五 OneNet物联网连接\onenet.png)

需要更改的是以下函数

**const char *ssid = "zhou";        //wifi名**

**const char *password = "12345678";    //wifi密码**

**const char *mqtt_server = "183.230.40.96"; //onenet 的 IP地址**

**const int port = 1883;           //端口号**

**\#define mqtt_devid "esp_device01" //设备ID**

**\#define mqtt_pubid "559349"     //产品ID**

**\#define mqtt_password "version=2018-10-31&res=products%2F559349%2Fdevices%2Fesp_device01&et=4092512761&method=md5&sign=RJkjLx1CPDnz%2Bx1qwfyuJA%3D%3D" //鉴权信息**

根据自己创建物联网平台的内容和设备号来更改

#### 程序源码

```
#include <Arduino.h>
#include "WiFi.h"
#include "PubSubClient.h"
#include "Ticker.h"
#include <DHT.h>
#include <DHT_U.h>
#define DHTPIN 16   
#define DHTTYPE DHT11  
DHT dht(DHTPIN, DHTTYPE);  

const char *ssid = "zhou";               //wifi名
const char *password = "12345678";       //wifi密码
const char *mqtt_server = "183.230.40.96"; //onenet 的 IP地址
const int port = 1883;                     //端口号

#define mqtt_devid "esp_device01" //设备ID
#define mqtt_pubid "559349"        //产品ID
//鉴权信息
#define mqtt_password "version=2018-10-31&res=products%2F559349%2Fdevices%2Fesp_device01&et=4092512761&method=md5&sign=RJkjLx1CPDnz%2Bx1qwfyuJA%3D%3D" //鉴权信息

WiFiClient espClient;           //创建一个WIFI连接客户端
PubSubClient client(espClient); // 创建一个PubSub客户端, 传入创建的WIFI客户端

char msgJson[75]; //发送信息缓冲区
//信息模板
char dataTemplate[] = "{\"id\":123,\"dp\":{\"temp\":[{\"v\":%.2f}],\"humi\":[{\"v\":%.2f}]}}";
Ticker tim1; //定时器,用来循环上传数据

//连接WIFI相关函数
void setupWifi()
{
  delay(10);
  Serial.println("连接WIFI");
  WiFi.begin(ssid, password);
  while (!WiFi.isConnected())
  {
    Serial.print(".");
    delay(500);
  }
  Serial.println("OK");
  Serial.println("Wifi连接成功");
}

//收到主题下发的回调, 注意这个回调要实现三个形参 1:topic 主题, 2: payload: 传递过来的信息 3: length: 长度
void callback(char *topic, byte *payload, unsigned int length)
{
  Serial.println("message rev:");
  Serial.println(topic);
  for (size_t i = 0; i < length; i++)
  {
    Serial.print((char)payload[i]);
  }
  Serial.println();
}

//向主题发送模拟的温湿度数据
void sendTempAndHumi()
{
  float h = dht.readHumidity();   
  float t = dht.readTemperature(); 
  if (client.connected())
  {
    snprintf(msgJson, 75, dataTemplate, t, h); //将模拟温湿度数据套入dataTemplate模板中, 生成的字符串传给msgJson
    Serial.print("public the data:");
    Serial.println(msgJson);
    client.publish("$sys/559349/esp_device01/dp/post/json", (uint8_t *)msgJson, strlen(msgJson));
    //发送数据到主题
  }
}

//重连函数, 如果客户端断线,可以通过此函数重连
void clientReconnect()
{
  while (!client.connected()) //再重连客户端
  {
    Serial.println("reconnect MQTT...");
    if (client.connect(mqtt_devid, mqtt_pubid, mqtt_password))
    {
      Serial.println("connected");
      client.subscribe("$sys/559349/esp_device01/cmd/request/#"); //订阅命令下发主题
    }
    else
    {
      Serial.println("failed");
      Serial.println(client.state());
      Serial.println("try again in 5 sec");
      delay(5000);
    }
  }
}

void setup()
{

  Serial.begin(115200);                                  //初始化串口
  delay(3000);                                           //这个延时是为了让我打开串口助手
  dht.begin(); 
  setupWifi();                                           //调用函数连接WIFI
  client.setServer(mqtt_server, port);                   //设置客户端连接的服务器,连接Onenet服务器, 使用6002端口
  client.connect(mqtt_devid, mqtt_pubid, mqtt_password); //客户端连接到指定的产品的指定设备.同时输入鉴权信息
  if (client.connected())
  {
    Serial.println("OneNet is connected!");//判断以下是不是连好了.
  }
  client.setCallback(callback);                                //设置好客户端收到信息是的回调
  client.subscribe("$sys/559349/esp_device01/cmd/request/#"); //订阅命令下发主题
  tim1.attach(10, sendTempAndHumi);                            //定时每10秒调用一次发送数据函数sendTempAndHumi
}

void loop()
{
  if (!WiFi.isConnected()) //先看WIFI是否还在连接
  {
    setupWifi();
  }
  if (!client.connected()) //如果客户端没连接ONENET, 重新连接
  {
    clientReconnect();
    delay(100);
  }
  client.loop(); //客户端循环检测
}
```

